% Run Hallin-Liska criterion to determine the number of factors in every 
% subpanel of volatilities over a frequency band [2pi/5 pi/252] 
% corresponding to oscillations with period [1 week, 2 years]

clear
clc
close all

addpath(genpath(pwd))

%%
DV=char('nkk225', 'sp500', 'speu350');
band=[2/5 2/(2*252)];

%%
c=input('Run Hallin Liska? \n (1) yes \n (2) no \n');
c=c==1;

%%
if c
    load('global_datiret_day.mat')
    
    panel = log( (ee+vv).^2 );
    
    clearvars -except panel c K DV pl band
    
    [T,N] = size(panel);
    w = floor(0.5*sqrt(T));
    n_j=201;
    n_us=336;
    n_eu=293;
    
    bl=[    1 n_j;  n_j+1 n_us+n_j;  n_us+n_j+1 n_us+n_j+n_eu];
    
end


%%
clc

K=size(DV,1);

for i=1:K
    
    sel=nchoosek(1:K,i);
    nco=size(sel,1);
    
    for j=1:nco
        
        nom=[];
        for ii=1:i
            nom=strcat(nom,DV(sel(j,ii),:),'_');
        end
        nom=nom(:,1:end-1);
        fprintf(' \n \n \t \t \t %s \n', nom)
        disp(repmat('_',1,40))
        
        %%
        if c
            subpanel=[];
            for jj=1:i
                subpanel = [subpanel    panel(:, bl(sel(j,jj),1):bl(sel(j,jj),2) ) ];
            end
            q_max=10;
            Ns=size(subpanel,2);
            c_max=1;
            cf=100;
            [nfact, v_nfact, cr] = numfactors_nonstd_log_band(subpanel, q_max, band, round(Ns/2), round(Ns/20), c_max,'p2', cf);
            title(['HL criterion: ',nom], 'Interpreter','none')
            close all
        else
            load nfact_vola

        end
        
        %% 
        if c
            eval(['nfact_',nom,'=nfact;'])
            eval(['v_nfact_',nom,'=v_nfact;'])
            eval(['cr_',nom,'=cr;'])
        end
        
        
        disp(repmat('=',1,40))
        
        %%
        if ~c
                c_max=1;
                cf=100;
                cr = (1:floor(c_max*cf))'/cf;
                eval(['nfact=nfact_',nom,';'])
                eval(['v_nfact=v_nfact_',nom,';'])
                figure
                plot(cr,5*v_nfact,'b-')
                hold all
                plot(cr,nfact,'r-')
                xlabel('c')
                axis tight
                legend('S_c','q^{*T}_{c;n}')
                title(['estimated number of factors - log criterion - ' nom])
                
        end
        
    end
end

clear nfact v_nfact cr q

if c
    save nfact_vola_3blocks_band q_* nfact_* v_nfact_* cr_* band
end
