% FHLZ - **** **** *****
% 
% X  - data matrix
% qq - should correspond to (q+1) in the paper
% k  - number of minimum lag for the polynomial A(L)
% w  - number of lag for the covariance matrix estimation
% nlagsimp
%
%   H(L)*x(t)=R*v(t)+H(L)*xsi(t)
%   chi(t)=C(L)v(t) --> H(L)*chi(t)=R*v(t) 
%                                   if C(L) is zeroless
%           Aj(L)*chi(t)=Rj*vj(t) for j=1:m
%

function [cstar,CL,v,C1,eta1] = fhlz_nstd2(X,qq,k,w,nlagsimp,idvar,nrepli)

T = size(X,1);
q = length(idvar);                                                         % number of shocks

sigma = std(X);
mu = mean(X);
z = (X - ones(T,1)*mu);
% ./(ones(T,1)*sigma);

covmat = ML_cestimate(z,q,w);                                              % cross and auto covariances of the common component
[n , o , h] = size(covmat);
H =(h+1)/2;
m = floor(n/qq);                                                            % n=(q+1)m


% chi= nan*ones(T-k,n,nrepli);
for h = 1:nrepli
%    h
    imp = nan*ones(n,q,nlagsimp);
    imp1 = nan*ones(n,q);
    uuu = nan*ones(T-k,q);
    riord = randperm(n);
    while not(isempty(intersect(idvar ,riord(qq*m+1:end))))
        riord = randperm(n);
    end    
    x = z(:,riord);         
    covmatrix = covmat(riord,riord,:);                                      % Permutation of the covariance matrix;
    kk = irfbic(covmatrix,q,k,T);
    [HL GL GL1 y]=StaticRepresentation(x,covmatrix,qq,m,nlagsimp,T,kk(1),H);        % Estimation of H(L) - G(L)=inv(H(L)) - w(t)
    S = diag(std(y)); yy = ML_center(y)*(S^-1); 
    yy1 = ML_center(y);
    %eta(:,:,h)=yy1;
    et{h}=yy1;    le(h)=length(et{h});
    opt.disp = 0; [V, MM] = eigs(cov(yy), q,'LM',opt);                      % MM = Lambda - V = P
    M = diag(sqrt(diag(MM)));                                               % M = sqrt(Lambda)   
    uu = yy*V*inv(M);                                                       % v(t)=sqrt(inv(Lambda))P'w(t)            
%     uuu=[zeros(nlagsimp-1,q); uu]; tau=size(uuu,1);
%     chih=zeros(tau,qq*m);            
    for lag = 1 : nlagsimp; 
        BBB(:, :, lag) = GL(:, :, lag)*S*V*M;         % C(L)     
%         chih = chih+uuu(:,:)*BBB(:,:, lag)';
    end;
%     chi(:,riord(1:qq*m),h) = chih(nlagsimp:T+nlagsimp-k-1,:);
    
%     imp(riord(1:qq*m),:,:) = BBB;                                           
%     imp1(riord(1:qq*m),:,:) = GL1*S*V*M; 
    imp(riord(1:n),:,:) = BBB;                                           
    imp1(riord(1:n),:,:) = GL1*S*V*M; 
    
%     [beta(:,:,:,h), u(:,:,h) beta1(:,:,h)] = ML_DfmCholIdent(imp,idvar,uu,imp1);        
    [beta(:,:,:,h), u0{h} beta1(:,:,h)] = ML_DfmCholIdent(imp,idvar,uu,imp1);        
    lu(h)=length(u0{h});
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %P1=S*V*M; P2=nan*ones(n,q); P2(riord(1:qq*m),:)=P1; P0=P2(idvar,:);
    %Pc = chol(P0*P0')';
    %Ph = inv(P0)*Pc;
    %for lag = 1 : nlagsimp ;Pbbb(:, :, lag) = BB(:, :, lag)*S*V*M*Ph;  end
    %Pbeta = nan*ones(n,q,nlagsimp); Pbeta(riord(1:qq*s),:,:) = Pbbb;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
end

% chat = nanmean(chi, 3); 
CL = nanmean(beta,4);

for i=1:nrepli
	u(:,:,i)=u0{i}(lu(i)-min(lu)+1:end,:);
    eta(:,:,i)=et{i}(le(i)-min(le)+1:end,:);
end
clear u0 et
v = nanmean(u,3);
eta1 = nanmean(eta,3);

C1 = nanmean(beta1,3);

vv=[zeros(nlagsimp-1,q); v];
cstar=zeros(n,T-k);
for ii=1:T-k;
    for jj=1:nlagsimp;
        cstar(:,ii)=cstar(:,ii)+CL(:,:,jj)*vv(ii+nlagsimp-jj,:)';
    end;
end;
cstar=cstar';

% bbeta = permute(beta,[4 2 3 1]);
% stdbeta = permute(nanstd(bbeta),[4 2 3 1]);
% cumbeta = CumImp(meanbeta,CodeData);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% =================================================================== %%%
%%% =================================================================== %%%

% VARcov estimate a VAR from autocovariance of the variables
% C - autoregressive coefficients 
% y - residuals

function [HL GL GL1 w]=StaticRepresentation(x,covmatrix,qq,m,nlagsimp,T,k,H)

n=size(x,2);

% GL = zeros(qq*m,qq*m,nlagsimp); HL = zeros(qq*m,qq*m,k+1); w = zeros(T-k,qq*m);
GL = zeros(n,n,nlagsimp); HL = zeros(n,n,k+1); w = zeros(T-k,n);

for mm = 1:m;                
    [~, CC B u]=VARcov(x,covmatrix,T,k,qq,mm,H,nlagsimp);                    % Estimation of Aj(L) - Gj(L) - vj(t)    
    HL((mm-1)*qq+1:qq*mm,(mm-1)*qq+1:qq*mm,:)=CC;                           % Building the H(L) Matrix
    w(:,(mm-1)*qq+1:qq*mm) = u;                                             % Residual w(t)   
    GL((mm-1)*qq+1:qq*mm,(mm-1)*qq+1:qq*mm,:)=B;                            % Building G(L)=inv(H(L))
    GL1((mm-1)*qq+1:qq*mm,(mm-1)*qq+1:qq*mm,:)=inv(sum(CC,3));              % long run impact matrix   
end

% last block of size reminder of n/(q+1) - if any
% ========================================================================================================================== %
if rem(n,qq)
    nlast = rem(n,qq);
    clear CC B u
    % [~, CC B u]=VARcov(x,covmatrix,T,k,qq,mm,H,nlagsimp);                    % Estimation of Aj(L) - Gj(L) - vj(t)
    % ================================================================= %
    A = zeros(nlast*k,nlast*k); B = zeros(nlast*k,nlast); xx = zeros(T-k,nlast*k);
    for j = 1:k
        for i = 1:k;
            A((j-1)*nlast+1:nlast*j,(i-1)*nlast+1:nlast*i) = covmatrix(qq*mm+1:n, qq*mm+1:n,H + i - j);
        end
        B((j-1)*nlast+1:nlast*j,:) = covmatrix(qq*mm+1:n, qq*mm+1:n,H - j);
        xx(:,(j-1)*nlast+1:nlast*j) =  x(k+1-j:T-j , qq*mm+1:n);
    end
    
    C = inv(A)*B;                                                               % Aj(L)
    u = x(k+1:T , qq*mm+1:n) - xx*C;                                            % Residual wj(t)
    CC(:,:,1) = eye(nlast); CC(:,:,2:k + 1) = - reshape(C',nlast,nlast,k);      % Reshaping Aj(L) s.t. it is invertible
    B = InvPolMatrix(CC,nlagsimp);
    % ================================================================= %
    
    HL(qq*mm+1:n,qq*mm+1:n,:)=CC;                           % Building the H(L) Matrix
    w(:,qq*mm+1:n) = u;                                     % Residual w(t)
    GL(qq*mm+1:n,qq*mm+1:n,:)=B;                            % Building G(L)=inv(H(L))
    GL1(qq*mm+1:n,qq*mm+1:n,:)=inv(sum(CC,3));              % long run impact matrix
    
end
% ========================================================================================================================== %

%%% =================================================================== %%%

%%% =================================================================== %%%
%%% =================================================================== %%%
 
% VARcov estimate a VAR from autocovariance of the variables
% C - autoregressive coefficients 
% B - Moving average Represetation
% u - residuals
function [C CC B u]=VARcov(x,covmatrix,T,k,qq,mm,H,nlagsimp)

A = zeros(qq*k,qq*k); B = zeros(qq*k,qq); xx = zeros(T-k,qq*k);
for j = 1:k
    for i = 1:k;    
        A((j-1)*qq+1:qq*j,(i-1)*qq+1:qq*i) = covmatrix((mm-1)*qq+1:mm*qq, (mm-1)*qq+1:mm*qq,H + i - j);   
    end    
    B((j-1)*qq+1:qq*j,:) = covmatrix((mm-1)*qq+1:mm*qq, (mm-1)*qq+1:mm*qq,H - j);     
    xx(:,(j-1)*qq+1:qq*j) =  x(k+1-j:T-j ,(mm-1)*qq+1:mm*qq);
end

C = inv(A)*B;                                                               % Aj(L)
u = x(k+1:T , (mm-1)*qq+1:mm*qq) - xx*C;                                    % Residual wj(t)
CC(:,:,1) = eye(qq); CC(:,:,2:k + 1) = - reshape(C',qq,qq,k);               % Reshaping Aj(L) s.t. it is invertible
B = InvPolMatrix(CC,nlagsimp);                                              % Building Gj(L)= inv(Aj(L))
%%% =================================================================== %%%

%%% =================================================================== %%%
%%% =================================================================== %%%

% inversion of a matrix of polynomials in the lag operator
function inverse = InvPolMatrix(poly,nlags)
n = size(poly,1); k = size(poly,3) - 1;
for s = 1:k+1; newpoly(:,:,s)=inv(poly(:,:,1))*poly(:,:,s); end;            % Guarantees that poly(:,:,1)=eye(n)
polynomialmatrix = - newpoly(:,:,2:k+1);
A = zeros(n*k,n*k);
A(n+1:n*k,1:n*(k-1)) = eye(n*(k-1));
for j = 1:k
   A(1:n,(j-1)*n+1:j*n) = polynomialmatrix(:,:,j);
end
inverse = zeros(n,n,nlags);
D = eye(n*k);
for j = 1:nlags 
   inverse(:,:,j) = D(1:n,1:n)*inv(poly(:,:,1));
   D = A*D;
end
%%% =================================================================== %%%

% n.b: covmatrix is a 3-dimensional object having the lags on the third
% dimension (the central lag is lag 0); q is the no. of shocks, K is the
% maximum VAR lag, T is the no. of time obs. pval is the probability of the 
% right tail 
function [bic, hq] = irfbic(covmatrix, q, K, T)
[n, ~, ~] = size(covmatrix);
qq = q  + 1;
s = floor(n/qq);
bic = zeros(s,K);
hq = bic;
 for k = 1:K
     for ss = 1:s-1
% compute relevant variances and covariances
[A, B] = yule(covmatrix((ss-1)*qq+1:ss*qq, (ss-1)*qq+1:ss*qq, :), k-1); 
% compute the VAR coefficients
C = A\B;
% compute the variance of the residuals
Ss = A(1:qq, 1:qq) - B'*C;
% compute the bic 
bic(ss,k) = log(det(Ss)) + (qq^2*k)*log(T)/T;
hq(ss,k) = log(det(Ss)) + (qq^2*k+qq)*2*log(log(T))/T;
     end
a = qq*(s-1);
nlast = n-a;
% repeat the steps above for the last group of variables which does not have size $q+1$
[A, B] = yule(covmatrix(a+1:n, a+1:n, :), k-1);  
C = A\B;
Ss = A(1:nlast, 1:nlast) - B'*C;
bic(s,k) = log(det(Ss)) + (nlast^2*k)*log(T)/T;
hq(s,k) = log(det(Ss)) + (nlast^2*k+nlast)*2*log(log(T))/T;
 end
% minimize bic and cbic 
 [~, bic] = min(bic,[],2);
  [~, hq] = min(hq,[],2);

 %%%%%%%%%%%%%%%%%%%%%%%%%%%
    function [S0, S1] = yule(covmatrix, k)
[n, ~, h] = size(covmatrix);
H = (h+1)/2;
covmat = covmatrix(:,:, H - k - 1:H + k + 1);
S0 = zeros(n*(k+1), n*(k+1));
striscia = reshape(covmat, n, n*(2*k+3));
for j = 1:k+1
    S0((j-1)*n+1:j*n,:) = striscia(:,(k+2-j)*n + 1:(2*k+3-j)*n);
end
  S1 = striscia(:,(k+2)*n + 1:(2*k+3)*n)'; 

