%-------------------------------------------------------------------
% [chi_yz, v_yz] = kestimate_2_blocks(y,z,q,qy,qz,M)
%-------------------------------------------------------------------
% computes the consequntive projections as suggested by Marc
%
% reference: Hallin, Liska : "GDFM in block structure" 
% code provided by Roman Liska
%
%
function  kestimate_3_blocks_SS(y1,y2,y3,q,q1,q2,q3,q12,q23,q13,M)

y1 = standardize(y1);
y2 = standardize(y2);
y3 = standardize(y3);


x = [y1,y2,y3];
[T, Nx] = size(x);


[T, Ny1] = size(y1);
[T, Ny2] = size(y2);
[T, Ny3] = size(y3);

w = M;
W = 2*w+1;

% compute covariances x
Sx = zeros(W,Nx*Nx);
for k = 1:w+1,
     Sk = (x(k:T,:))'*(x(1:T+1-k,:))/(T-k);
     S_k = Sk';
     Sx(w-k+2,:) = S_k(:)'; 
     Sx(w+k,:) = Sk(:)';     
end
% compute the spectral matrix in W points (S)
Factor = exp(-sqrt(-1)*(-w:w)'*(0:2*pi/W:4*pi*w/W)); %  (2w+1) x  (2w+1) 
Sx = diag(triang(W))*Sx(1:W,:);
Sxx = reshape(Sx'*Factor,Nx,Nx*W);

% compute the first nfacors eigenvalues and 
% egenvectors  for all points (D, A) and the K function
Pa1 = zeros(Ny1,Nx,W);
Pa2 = zeros(Ny2,Nx,W);


Sx = Sxx(:,1:Nx);

% projection furter 1st block  on q principal components of 3 blocks

i1 = [1:Ny1];
i2 = [Ny1+1:Ny1+Ny2];
i3 = [Ny1+Ny2+1:Ny1+Ny2+Ny3];


for j = 1:w+1,

    Sx = Sxx(:,(j-1)*Nx+1:j*Nx);
    [Ax,Dx] = eigs(Sx, q);

    Dx = diag(diag(real(Dx)));
    Dxi = diag(diag(real(Dx)).^(-1));
    
    % 1st projection
    Pa1(:,:,j) = Ax(i1,:)*Ax';
    Pa2(:,:,j) = Ax(i2,:)*Ax';
    Pa3(:,:,j) = Ax(i3,:)*Ax';
    
    Ka1(:,j) = real(diag(Ax(i1,:)*Dx*Ax(i1,:)'));
    Ka2(:,j) = real(diag(Ax(i2,:)*Dx*Ax(i2,:)'));
    Ka3(:,j) = real(diag(Ax(i3,:)*Dx*Ax(i3,:)'));

    % projection furter 1st block  on q principal components of 3 blocks

    [A12,D12] = eigs(Ax([i1,i2],:)*Dx*Ax([i1,i2],:)', q12);
    [A13,D13] = eigs(Ax([i1,i3],:)*Dx*Ax([i1,i3],:)', q13);
    [A23,D23] = eigs(Ax([i2,i3],:)*Dx*Ax([i2,i3],:)', q23);
    
    
    D12 = diag(diag(real(D12)));
    D13 = diag(diag(real(D13)));
    D23 = diag(diag(real(D23)));

    D12i = diag(diag(real(D12)).^(-1));
    D13i = diag(diag(real(D13)).^(-1));
    D23i = diag(diag(real(D23)).^(-1));

    D12i2 = diag(diag(real(D12)).^(-0.5));
    D13i2 = diag(diag(real(D13)).^(-0.5));
    D23i2 = diag(diag(real(D23)).^(-0.5));
    
    
    Pb12s1(:,:,j) =  A12(1:Ny1,:)*A12'*Ax([i1,i2],:)*Ax';
    Pb12s2(:,:,j) =  A12(Ny1+1:end,:)*A12'*Ax([i1,i2],:)*Ax';

    Pb13s1(:,:,j) =  A13(1:Ny1,:)*A13'*Ax([i1,i3],:)*Ax';
    Pb13s3(:,:,j) =  A13(Ny1+1:end,:)*A13'*Ax([i1,i3],:)*Ax';

    Pb23s2(:,:,j) =  A23(1:Ny2,:)*A23'*Ax([i2,i3],:)*Ax';
    Pb23s3(:,:,j) =  A23(Ny2+1:end,:)*A23'*Ax([i2,i3],:)*Ax';
    

    Kb12s1(:,j) = real(diag(A12(1:Ny1,:)*D12*A12(1:Ny1,:)'));
    Kb12s2(:,j) = real(diag(A12(Ny1+1:end,:)*D12*A12(Ny1+1:end,:)'));

    Kb13s1(:,j) = real(diag(A13(1:Ny1,:)*D13*A13(1:Ny1,:)'));
    Kb13s3(:,j) = real(diag(A13(Ny1+1:end,:)*D13*A13(Ny1+1:end,:)'));

    Kb23s2(:,j) = real(diag(A23(1:Ny2,:)*D23*A23(1:Ny2,:)'));
    Kb23s3(:,j) = real(diag(A23(Ny2+1:end,:)*D23*A23(Ny2+1:end,:)'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 
    [A121,D121] = eigs(A12(1:Ny1,:)*D12*A12(1:Ny1,:)', q1);
    [A122,D122] = eigs(A12(Ny1+1:end,:)*D12*A12(Ny1+1:end,:)', q2);
    
    [A131,D131] = eigs(A13(1:Ny1,:)*D13*A13(1:Ny1,:)', q1);
    [A133,D133] = eigs(A13(Ny1+1:end,:)*D13*A13(Ny1+1:end,:)', q3);

    [A232,D232] = eigs(A23(1:Ny2,:)*D23*A23(1:Ny2,:)', q2);
    [A233,D233] = eigs(A23(Ny2+1:end,:)*D23*A23(Ny2+1:end,:)', q3);
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    Pc121(:,:,j) =  A121*A121'*A12(1:Ny1,:)*A12'*Ax([i1,i2],:)*Ax';
    Pc122(:,:,j) =  A122*A122'*A12(Ny1+1:end,:)*A12'*Ax([i1,i2],:)*Ax';

    Pc131(:,:,j) =  A131*A131'*A13(1:Ny1,:)*A13'*Ax([i1,i3],:)*Ax';
    Pc133(:,:,j) =  A133*A133'*A13(Ny1+1:end,:)*A13'*Ax([i1,i3],:)*Ax';

    Pc232(:,:,j) =  A232*A232'*A23(1:Ny2,:)*A23'*Ax([i2,i3],:)*Ax';
    Pc233(:,:,j) =  A233*A233'*A23(Ny2+1:end,:)*A23'*Ax([i2,i3],:)*Ax';

    
    Kc121(:,j) = real(diag(A121*D121*A121'));
    Kc122(:,j) = real(diag(A122*D122*A122'));

    Kc131(:,j) = real(diag(A131*D131*A131'));
    Kc133(:,j) = real(diag(A133*D133*A133'));

    Kc232(:,j) = real(diag(A232*D232*A232'));
    Kc233(:,j) = real(diag(A233*D233*A233'));


 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  12
 %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    D121i2 = diag(diag(real(D121)).^(-0.5));
    D122i = diag(diag(real(D122)).^(-1));
 
     SV12 = D121i2*(A121'*A12(1:Ny1,:)*A12'*Ax([i1,i2],:)*Dx*(  A122'*A12(Ny1+1:end,:)*A12'*Ax([i1,i2],:)  )')*...
         D122i*...
         (D121i2*(A121'*A12(1:Ny1,:)*A12'*Ax([i1,i2],:)*Dx*(  A122'*A12(Ny1+1:end,:)*A12'*Ax([i1,i2],:)  )'))';
     
     if q1+q2-q12
         
         [A5,D5] = eigs(SV12, q1+q2-q12);
         
         if abs(sum(sum(A5'*A5-eye(q1+q2-q12)))) > 1e-05             
             A5 = eye(q1+q2-q12);
         end
         
         Base = A5'*D121i2*(A121'*A12(1:Ny1,:)*A12'*Ax([i1,i2],:));
         Base12 = Base*Ax';
         
         
         Pd1w12(:,:,j) = A121*A121'*A12(1:Ny1,:)*A12'*Ax([i1,i2],:)*Dx*...
             (A121'*A12(1:Ny1,:)*A12'*Ax([i1,i2],:))'*D121i2'*A5*...
             Base12;
         
         
         Pd2w12(:,:,j) = A122*A122'*A12(Ny1+1:end,:)*A12'*Ax([i1,i2],:)*Dx*...
             (A121'*A12(1:Ny1,:)*A12'*Ax([i1,i2],:))'*D121i2'*A5*...
             Base12;
         
         
         
         tmp = A121*A121'*A12(1:Ny1,:)*A12'*Ax([i1,i2],:)*Dx*...
             (A121'*A12(1:Ny1,:)*A12'*Ax([i1,i2],:))'*D121i2'*A5*...
             Base;
         Kd1w12(:,j) = real(diag(tmp*Dx*tmp'));
         
         
         tmp = A122*A122'*A12(Ny1+1:end,:)*A12'*Ax([i1,i2],:)*Dx*...
             (A121'*A12(1:Ny1,:)*A12'*Ax([i1,i2],:))'*D121i2'*A5*...
             Base;
         Kd2w12(:,j) = real(diag(tmp*Dx*tmp'));
         
     else
         
         Kd2w12 =[];
         Kd1w12 =[];
         
         
     end
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  13
 %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    D131i2 = diag(diag(real(D131)).^(-0.5));
    D133i = diag(diag(real(D133)).^(-1));
 
     SV13 = D131i2*(A131'*A13(1:Ny1,:)*A13'*Ax([i1,i3],:)*Dx*(  A133'*A13(Ny1+1:end,:)*A13'*Ax([i1,i3],:)  )')*...
         D133i*...
         (D131i2*(A131'*A13(1:Ny1,:)*A13'*Ax([i1,i3],:)*Dx*(  A133'*A13(Ny1+1:end,:)*A13'*Ax([i1,i3],:)  )'))';

     if q1+q3-q13
     
    [A5,D5] = eigs(SV13, q1+q3-q13);
    
    if abs(sum(sum(A5'*A5-eye(q1+q3-q13)))) > 1e-05
      A5 = eye(q1+q3-q13);
    end
    
    Base = A5'*D131i2*(A131'*A13(1:Ny1,:)*A13'*Ax([i1,i3],:)); 
    
  
             
 tmp = A131*A131'*A13(1:Ny1,:)*A13'*Ax([i1,i3],:)*Dx*...
                  (A131'*A13(1:Ny1,:)*A13'*Ax([i1,i3],:))'*D131i2'*A5*...
                 Base;
  Kd1w13(:,j) = real(diag(tmp*Dx*tmp'));         
       
    
  tmp = A133*A133'*A13(Ny1+1:end,:)*A13'*Ax([i1,i3],:)*Dx*...
                     (A131'*A13(1:Ny1,:)*A13'*Ax([i1,i3],:))'*D131i2'*A5*...
                 Base;
  Kd3w13(:,j) = real(diag(tmp*Dx*tmp'));         
  
  
     else
         Kd3w13 =[];
         Kd1w13 = [];
         
         
     end
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  23
 %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    D232i2 = diag(diag(real(D232)).^(-0.5));
    D233i = diag(diag(real(D233)).^(-1));
 
     SV23 = D232i2*(A232'*A23(1:Ny2,:)*A23'*Ax([i2,i3],:)*Dx*(  A233'*A23(Ny2+1:end,:)*A23'*Ax([i2,i3],:)  )')*...
         D233i*...
         (D232i2*(A232'*A23(1:Ny2,:)*A23'*Ax([i2,i3],:)*Dx*(  A233'*A23(Ny2+1:end,:)*A23'*Ax([i2,i3],:)  )'))';

     
%     [A5,D5] = eigs(SV23, q2+q3-q23);
    
    if q2+q3-q23
        [A5,D5] = eigs(SV23, q2+q3-q23);
%     if abs(sum(sum(A5'*A5-eye(q2+q3-q23)))) > 1e-05
%       A5 = eye(q2+q3-q23);
%     end
    
    Base = A5'*D232i2*(A232'*A23(1:Ny2,:)*A23'*Ax([i2,i3],:)); 
    
  
 tmp = A232*A232'*A23(1:Ny2,:)*A23'*Ax([i2,i3],:)*Dx*...
                  (A232'*A23(1:Ny2,:)*A23'*Ax([i2,i3],:))'*D232i2'*A5*...
                 Base;
  Kd2w23(:,j) = real(diag(tmp*Dx*tmp'));         
       
    
  tmp = A233*A233'*A23(Ny2+1:end,:)*A23'*Ax([i2,i3],:)*Dx*...
                     (A232'*A23(1:Ny2,:)*A23'*Ax([i2,i3],:))'*D232i2'*A5*...
                 Base;
  Kd3w23(:,j) = real(diag(tmp*Dx*tmp'));         
 
    else
        
        Kd2w23 =[];
        Kd3w23 =[];
        
    end

%%
if j>1

    Pa1(:,:,W+2-j) = conj(Pa1(:,:,j));
    Pa2(:,:,W+2-j) = conj(Pa2(:,:,j));
    Pa3(:,:,W+2-j) = conj(Pa3(:,:,j));

    Pb12s1(:,:,W+2-j) = conj(Pb12s1(:,:,j));
    Pb12s2(:,:,W+2-j) = conj(Pb12s2(:,:,j));
    Pb13s1(:,:,W+2-j) = conj(Pb13s1(:,:,j));
    Pb13s3(:,:,W+2-j) = conj(Pb13s3(:,:,j));
    Pb23s2(:,:,W+2-j) = conj(Pb23s2(:,:,j));
    Pb23s3(:,:,W+2-j) = conj(Pb23s3(:,:,j));
    
    Pc121(:,:,W+2-j) = conj(Pc121(:,:,j));
    Pc122(:,:,W+2-j) = conj(Pc122(:,:,j));
    Pc131(:,:,W+2-j) = conj(Pc131(:,:,j));
    Pc133(:,:,W+2-j) = conj(Pc133(:,:,j));
    Pc232(:,:,W+2-j) = conj(Pc232(:,:,j));
    Pc233(:,:,W+2-j) = conj(Pc233(:,:,j));

    
    Ka1(:,W+2-j) =  Ka1(:,j);
    Ka2(:,W+2-j) =  Ka2(:,j);
    Ka3(:,W+2-j) =  Ka3(:,j);

    Kb12s1(:,W+2-j) = Kb12s1(:,j);
    Kb12s2(:,W+2-j) = Kb12s2(:,j);
    Kb13s1(:,W+2-j) = Kb13s1(:,j);
    Kb13s3(:,W+2-j) = Kb13s3(:,j);
    Kb23s2(:,W+2-j) = Kb23s2(:,j);
    Kb23s3(:,W+2-j) = Kb23s3(:,j);

    Kc121(:,W+2-j) = Kc121(:,j);
    Kc122(:,W+2-j) = Kc122(:,j);
    Kc131(:,W+2-j) = Kc131(:,j);
    Kc133(:,W+2-j) = Kc133(:,j);
    Kc232(:,W+2-j) = Kc232(:,j);
    Kc233(:,W+2-j) = Kc233(:,j);
    
    
    if q1+q2-q12      
        Kd1w12(:,W+2-j) = Kd1w12(:,j);
        Kd2w12(:,W+2-j) = Kd2w12(:,j);    
    end
    
    
    if q1+q3-q13    
        Kd1w13(:,W+2-j) = Kd1w13(:,j);         
        Kd3w13(:,W+2-j) = Kd3w13(:,j);
    end

    
    Kd2w23(:,W+2-j) = Kd2w23(:,j);         
    Kd3w23(:,W+2-j) = Kd3w23(:,j);         
    
end

end

disp(['----------------------------------------------------------------------------------------------------'])

tmp = [Ka1, Ka1(:,1)];
disp(['explained y1:123 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny1*2*pi) ) ,' % '])
tmp = [Kb12s1, Kb12s1(:,1)];
disp(['explained y1:123-12 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny1*2*pi) ) ,' % '])
tmp = [Kc121, Kc121(:,1)];
disp(['explained y1:123-12-1 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny1*2*pi) ) ,' % '])
tmp = [Kb13s1, Kb13s1(:,1)];
disp(['explained y1:123-13 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny1*2*pi) ),' % '])
tmp = [Kc131, Kc131(:,1)];
disp(['explained y1:123-13-1 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny1*2*pi) ),' % '])

if q1+q2-q12
    tmp = [Kd1w12, Kd1w12(:,1)];
    disp(['explained y1:123-12-1-1x2 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny1*2*pi) ),' % '])
else
    disp(['explained y1:123-12-1-1x2 :  empty intersection 1-2!'])
end

if q1+q3-q13
    tmp = [Kd1w13, Kd1w13(:,1)];
    disp(['explained y1:123-13-1-1x3 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny1*2*pi) ),' % '])
else
    disp(['explained y1:123-13-1-1x3 :  empty intersection 1-3!'])
end

disp(['----------------------------------------------------------------------------------------------------'])

tmp = [Ka2, Ka2(:,1)];
disp(['explained y2:123 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny2*2*pi) ),' % '])
tmp = [Kb12s2, Kb12s2(:,1)];
disp(['explained y2:123-12 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny2*2*pi) ),' % '])
tmp = [Kc122, Kc122(:,1)];
disp(['explained y2:123-12-2 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny2*2*pi) ),' % '])
tmp = [Kb23s2, Kb23s2(:,1)];
disp(['explained y2:123-23 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny2*2*pi) ),' % '])
tmp = [Kc232, Kc232(:,1)];
disp(['explained y2:123-23-2 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny2*2*pi) ),' % '])

if q1+q2-q12
    tmp = [Kd2w12, Kd2w12(:,1)];
    disp(['explained y2:123-12-2-1x2 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny2*2*pi) ),' % '])
else
    disp(['explained y2:123-12-2-1x2 :  empty intersection 1-2!'])
end

tmp = [Kd2w23, Kd2w23(:,1)];
disp(['explained y2:123-23-2-2x3 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny2*2*pi) ),' % '])


disp(['----------------------------------------------------------------------------------------------------'])

tmp = [Ka3, Ka3(:,1)];
disp(['explained y3:123 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny3*2*pi) ),' % '])
tmp = [Kb13s3, Kb13s3(:,1)];
disp(['explained y3:123-13 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny3*2*pi) ),' % '])
tmp = [Kc133, Kc133(:,1)];
disp(['explained y3:123-13-3 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny3*2*pi) ),' % '])
tmp = [Kb23s3, Kb23s3(:,1)];
disp(['explained y3:123-23 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny3*2*pi) ),' % '])
tmp = [Kc233, Kc233(:,1)];
disp(['explained y3:123-23-3 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny3*2*pi) ),' % '])

if q1+q3-q13
    tmp = [Kd3w13, Kd3w13(:,1)];
    disp(['explained y3:123-13-3-1x3 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny3*2*pi) ),' % '])
else
    disp(['explained y3:123-13-3-1x3 :  empty intersection 1-3!'])
end

tmp = [Kd3w23, Kd3w23(:,1)];
disp(['explained y3:123-23-3-2x3 variation by EIGS method : ', num2str(100*((sum(sum(real(tmp(:,1:end-1)))) + sum(sum(real(tmp(:,2:end)))))/2*(2*pi/W))/(Ny3*2*pi) ),' % '])

disp(['----------------------------------------------------------------------------------------------------'])
